@@ -196,14 +196,14 @@ if(run.apsimx.edit){
   
 }
 
-## Testing edit when node = "Other"
+#### Testing edit when node = "Other" ----
 
 if(run.apsimx.edit){
   
   extd.dir <- system.file("extdata", package = "apsimx")
   
   pp <- inspect_apsimx("Wheat.apsimx", src.dir = extd.dir, node = "Manager",
-                       parm = list("SowingFert", NA))
+                       parm = list("Fert", NA))
   
   edit_apsimx("Wheat.apsimx", node = "Other",
               src.dir = extd.dir, wrt.dir = tmp.dir,
@@ -211,10 +211,28 @@ if(run.apsimx.edit){
               overwrite = TRUE)
 
   inspect_apsimx("Wheat.apsimx", src.dir = tmp.dir, node = "Manager",
-                 parm = list("SowingFertiliser", NA))  
+                 parm = list("Fert", NA))  
   
   file.copy(file.path(tmp.dir, "Wheat.apsimx"), ".")
   
+  #### Testing edit when node == "Other" and there are multiple simulations
+  ex.dir <- auto_detect_apsimx_examples()
+  file.copy(file.path(ex.dir, "AgPasture.apsimx"), tmp.dir)
+  
+  inspect_apsimx("AgPasture.apsimx", src.dir = tmp.dir, node = "Other")
+  inspect_apsimx("AgPasture.apsimx", src.dir = tmp.dir, node = "Other", parm = 2)
+  
+  edit_apsimx("AgPasture.apsimx", src.dir = tmp.dir, 
+              node = "Other",
+              parm.path = ".Simulations.AgPastureExample.Clock.Start",
+              value = "2024-12-31T00:00:00")
+  
+  inspect_apsimx("AgPasture-edited.apsimx", src.dir = tmp.dir, 
+                 root = list("AgPastureExample"))
+  ### It might have worked?
+  file.remove(file.path(tmp.dir, "AgPasture-edited.apsimx"))
+  
+  
 }
 
 ## Testing changing variables in the outputfile
